@@ -1,7 +1,7 @@
 /*
  *  
  *  GeanyGenDoc, a Geany plugin to ease generation of source code documentation
- *  Copyright (C) 2010  Colomban Wendling <ban@herbesfolles.org>
+ *  Copyright (C) 2010-2011  Colomban Wendling <ban@herbesfolles.org>
  *  
  *  This program is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -81,7 +81,9 @@ static PluginData plugin_data = {
   NULL, 0, NULL, NULL, NULL, 0l
 };
 
-/* global plugin options */
+/* global plugin options
+ * default values that needs to be set dynamically goes at the top of
+ * load_configuration() */
 gchar      *GGD_OPT_doctype[GEANY_MAX_BUILT_IN_FILETYPES] = { NULL };
 gboolean    GGD_OPT_save_to_refresh                       = FALSE;
 gboolean    GGD_OPT_indent                                = TRUE;
@@ -187,6 +189,9 @@ load_configuration (void)
   GError   *err = NULL;
   guint     i;
   
+  /* default options that needs to be set dynamically */
+  GGD_OPT_doctype[0] = g_strdup ("doxygen");
+  
   plugin->config = ggd_opt_group_new ("General");
   ggd_opt_group_add_string (plugin->config, &GGD_OPT_doctype[0], "doctype");
   for (i = 1; i < GEANY_MAX_BUILT_IN_FILETYPES; i++) {
@@ -226,8 +231,8 @@ load_configuration (void)
 static void
 unload_configuration (void)
 {
-  gchar    *conffile;
-  GError   *err = NULL;
+  gchar  *conffile;
+  GError *err = NULL;
   
   conffile = ggd_get_config_file ("ggd.conf", NULL, GGD_PERM_RW, &err);
   if (conffile) {
@@ -299,19 +304,57 @@ open_current_filetype_conf_handler (GtkWidget  *widget,
   
   doc = document_get_current ();
   if (DOC_VALID (doc)) {
-    gchar  *path;
+    gchar  *path_read;
+    gchar  *path_write;
     GError *err = NULL;
     
-    path = ggd_file_type_manager_get_conf_path (doc->file_type->id,
-                                                GGD_PERM_R | GGD_PERM_W, &err);
-    if (! path) {
+    path_write = ggd_file_type_manager_get_conf_path (doc->file_type->id,
+                                                      GGD_PERM_W | GGD_PERM_NOCREAT,
+                                                      &err);
+    if (! path_write) {
       msgwin_status_add (_("Failed to find configuration file "
                            "for file type \"%s\": %s"),
                          doc->file_type->name, err->message);
       g_error_free (err);
     } else {
-      document_open_file (path, FALSE, NULL, NULL);
-      g_free (path);
+      gchar *text = NULL;
+      gchar *path_write_u8;
+      
+      path_read = ggd_file_type_manager_get_conf_path (doc->file_type->id,
+                                                       GGD_PERM_R, &err);
+      if (! path_read) {
+        text = g_strdup (_(
+          "# Configuration for this file type doesn't exist yet.\n"
+          "# To create it, just write it in this file and save it. For the description\n"
+          "# of the syntax of this file, please refer to the manual.\n"
+        ));
+      } else {
+        gchar  *content = NULL;
+        gsize   length;
+        
+        if (! g_file_get_contents (path_read, &content, &length, &err)) {
+          gchar *display_path_read;
+          
+          display_path_read = g_filename_display_name (path_read);
+          g_warning (_("Failed to load file \"%s\": %s"),
+                     display_path_read, err->message);
+          g_free (display_path_read);
+          g_error_free (err);
+        } else {
+          text = encodings_convert_to_utf8 (content, length, NULL);
+          g_free (content);
+        }
+        g_free (path_read);
+      }
+      path_write_u8 = utils_get_utf8_from_locale (path_write);
+      /* It's no Ruby, but it is the closest one I've found. It has:
+       *  - # comments
+       *  - multi-line double-quoted strings
+       */
+      document_new_file (path_write_u8, filetypes[GEANY_FILETYPES_RUBY], text);
+      g_free (path_write_u8);
+      g_free (text);
+      g_free (path_write);
     }
   }
 }
