@@ -1,7 +1,7 @@
 /*
  *  
  *  GeanyGenDoc, a Geany plugin to ease generation of source code documentation
- *  Copyright (C) 2009-2010  Colomban Wendling <ban@herbesfolles.org>
+ *  Copyright (C) 2009-2011  Colomban Wendling <ban@herbesfolles.org>
  *  
  *  This program is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -54,12 +54,14 @@ parser_parse_to_string (const CtplToken *tree,
                         CtplEnviron     *env,
                         GError         **error)
 {
-  GOutputStream  *ostream;
-  gchar          *output = NULL;
+  GOutputStream    *gostream;
+  CtplOutputStream *ostream;
+  gchar            *output = NULL;
   
-  ostream = g_memory_output_stream_new (NULL, 0, g_try_realloc, NULL);
+  gostream = g_memory_output_stream_new (NULL, 0, g_try_realloc, NULL);
+  ostream = ctpl_output_stream_new (gostream);
   if (ctpl_parser_parse (tree, env, ostream, error)) {
-    GMemoryOutputStream  *memostream = G_MEMORY_OUTPUT_STREAM (ostream);
+    GMemoryOutputStream  *memostream = G_MEMORY_OUTPUT_STREAM (gostream);
     gsize                 size;
     gsize                 data_size;
     
@@ -84,8 +86,9 @@ parser_parse_to_string (const CtplToken *tree,
     if (size > data_size) {
       output[data_size] = 0;
     }
-    g_object_unref (ostream);
   }
+  ctpl_output_stream_unref (ostream);
+  g_object_unref (gostream);
   
   return output;
 }
@@ -347,7 +350,9 @@ do_insert_comment (GeanyDocument   *doc,
         line = adjust_start_line (sci, tag_array, tag, line);
         pos = sci_get_position_from_line (sci, line);
         if (GGD_OPT_indent) {
-          pos += sci_get_line_indentation (sci, line);
+          while (isspace (sci_get_char_at (sci, pos))) {
+            pos++;
+          }
         }
         break;
       }
@@ -514,6 +519,8 @@ ggd_insert_comment (GeanyDocument  *doc,
   
   g_return_val_if_fail (DOC_VALID (doc), FALSE);
   
+ again:
+  
   if (doc->tm_file) {
     tag_array = doc->tm_file->tags_array;
     tag = ggd_tag_find_from_line (tag_array, line + 1 /* it is a SCI line */);
@@ -526,6 +533,12 @@ ggd_insert_comment (GeanyDocument  *doc,
       GList          *tag_list = NULL;
       
       setting = get_setting_from_tag (doctype, doc, tag, &tag);
+      if (setting && setting->policy == GGD_POLICY_PASS) {
+        /* We want to completely skip this tag, so try previous line instead
+         * FIXME: this implementation is kinda ugly... */
+        line--;
+        goto again;
+      }
       if (setting && setting->autodoc_children) {
         tag_list = ggd_tag_find_children_filtered (tag_array, tag,
                                                    FILETYPE_ID (doc->file_type),
@@ -562,10 +575,7 @@ ggd_insert_all_comments (GeanyDocument *doc,
   g_return_val_if_fail (DOC_VALID (doc), FALSE);
   
   if (! doc->tm_file) {
-    /* FIXME: we're on string freeze then don't add a new string; simply re-use
-     * an existing one. Need to fix this after string freeze. */
-    /*msgwin_status_add (_("No tags in the document"));*/
-    msgwin_status_add (_("No valid tag at line %d."), 1);
+    msgwin_status_add (_("No tags in the document"));
   } else if (get_config (doc, doc_type, &filetype, &doctype)) {
     GList    *tag_list;
     
