@@ -1,7 +1,7 @@
  /*
  *      geanylatex.h
  *
- *      Copyright 2008-2010 Frank Lanitz <frank(at)frank(dot)uvena(dot)de>
+ *      Copyright 2008-2011 Frank Lanitz <frank(at)frank(dot)uvena(dot)de>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -39,11 +39,9 @@
 #include "latexstructure.h"
 #include "latexkeybindings.h"
 
-#ifdef HAVE_LOCALE_H
-# include <locale.h>
-#endif
 
 #include <string.h>
+
 typedef void (*MenuCallback) (G_GNUC_UNUSED GtkMenuItem * menuitem, G_GNUC_UNUSED gpointer gdata);
 
 
@@ -69,9 +67,10 @@ void glatex_insert_label_activated(G_GNUC_UNUSED GtkMenuItem * menuitem,
 	 G_GNUC_UNUSED gpointer gdata);
 void glatex_insert_ref_activated(G_GNUC_UNUSED GtkMenuItem * menuitem,
 	 G_GNUC_UNUSED gpointer gdata);
-void glatex_replace_special_character();
+void glatex_replace_special_character(void);
 void glatex_insert_usepackage_dialog(G_GNUC_UNUSED GtkMenuItem * menuitem,
 	 G_GNUC_UNUSED gpointer gdata);
 void glatex_insert_command_activated(G_GNUC_UNUSED GtkMenuItem * menuitem,
 	 G_GNUC_UNUSED gpointer gdata);
+void on_insert_bibtex_dialog_activate(GtkMenuItem * menuitem, gpointer gdata);
 #endif
