@@ -1,7 +1,7 @@
 /*
  *	  latexenvironments.c
  *
- *	  Copyright 2009-2010 Frank Lanitz <frank(at)frank(dot)uvena(dot)de>
+ *	  Copyright 2009-2011 Frank Lanitz <frank(at)frank(dot)uvena(dot)de>
  *
  *	  This program is free software; you can redistribute it and/or modify
  *	  it under the terms of the GNU General Public License as published by
@@ -60,7 +60,7 @@ gchar *glatex_list_environments [] =
 
 
 /* if type == -1 then we will try to autodetect the type */
-void glatex_insert_environment(gchar *environment, gint type)
+void glatex_insert_environment(const gchar *environment, gint type)
 {
 	GeanyDocument *doc = NULL;
 
