@@ -1,7 +1,7 @@
 /*
  *      latexenvironments.h
  *
- *      Copyright 2009-2010 Frank Lanitz <frank(at)frank(dot)uvena(dot)de>
+ *      Copyright 2009-2011 Frank Lanitz <frank(at)frank(dot)uvena(dot)de>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -50,7 +50,7 @@ extern SubMenuTemplate glatex_environment_array[];
 
 extern CategoryName glatex_environment_cat_names[];
 
-void glatex_insert_environment(gchar *environment, gint type);
+void glatex_insert_environment(const gchar *environment, gint type);
 
 void
 glatex_insert_environment_dialog(G_GNUC_UNUSED GtkMenuItem *menuitem,
