@@ -1,7 +1,7 @@
 /*
  *      latexkeybindings.h
  *
- *      Copyright 2009-2010 Frank Lanitz <frank(at)frank(dot)uvena(dot)de>
+ *      Copyright 2009-2011 Frank Lanitz <frank(at)frank(dot)uvena(dot)de>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -48,6 +48,7 @@ enum
 	KB_LATEX_STRUCTURE_LVLDOWN,
 	KB_LATEX_USEPACKAGE_DIALOG,
 	KB_LATEX_INSERT_COMMAND,
+	KB_LATEX_INSERT_CITE,
 	COUNT_KB
 };
 
@@ -72,5 +73,6 @@ void glatex_kb_structure_lvlup(G_GNUC_UNUSED guint key_id);
 void glatex_kb_structure_lvldown(G_GNUC_UNUSED guint key_id);
 void glatex_kb_usepackage_dialog(G_GNUC_UNUSED guint key_id);
 void glatex_kb_insert_command_dialog(G_GNUC_UNUSED guint key_id);
+void glatex_kb_insert_bibtex_cite(G_GNUC_UNUSED guint key_id);
 
 #endif
