@@ -1,7 +1,7 @@
 /*
  *      geanysendmail.c
  *
- *      Copyright 2007-2010 Frank Lanitz <frank(at)frank(dot)uvena(dot)de>
+ *      Copyright 2007-2011 Frank Lanitz <frank(at)frank(dot)uvena(dot)de>
  *      Copyright 2007 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
  *      Copyright 2007, 2008 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *      Copyright 2008, 2009 Timothy Boronczyk <tboronczyk(at)gmail(dot)com>
@@ -35,10 +35,16 @@ GeanyPlugin		*geany_plugin;
 GeanyData		*geany_data;
 GeanyFunctions	*geany_functions;
 
-PLUGIN_VERSION_CHECK(147)
+PLUGIN_VERSION_CHECK(199)
 
-PLUGIN_SET_INFO(_("GeanySendMail"), _("A little plugin to send the current \
-file as attachment by user's favorite mailer"), VERSION, "Frank Lanitz <frank@frank.uvena.de>")
+PLUGIN_SET_TRANSLATABLE_INFO(
+	LOCALEDIR,
+	GETTEXT_PACKAGE,
+	_("GeanySendMail"),
+	_("A little plugin to send the current "
+	  "file as attachment by user's favorite mailer"),
+	VERSION,
+	"Frank Lanitz <frank@frank.uvena.de>")
 
 /* Keybinding(s) */
 enum
@@ -47,8 +53,6 @@ enum
 	COUNT_KB
 };
 
-PLUGIN_KEY_GROUP(sendmail, COUNT_KB)
-
 static gchar *config_file = NULL;
 static gchar *mailer = NULL;
 static gchar *address = NULL;
@@ -59,12 +63,6 @@ GtkWidget *mailbutton = NULL;
 static GtkWidget *main_menu_item = NULL;
 
 
-static void on_enter_key_pressed_in_entry(G_GNUC_UNUSED GtkWidget *widget, gpointer dialog )
-{
-	gtk_dialog_response(GTK_DIALOG(dialog), GTK_RESPONSE_ACCEPT);
-}
-
-
 /* Callback for sending file as attachment */
 static void
 send_as_attachment(G_GNUC_UNUSED GtkMenuItem *menuitem, G_GNUC_UNUSED gpointer gdata)
@@ -74,10 +72,6 @@ send_as_attachment(G_GNUC_UNUSED GtkMenuItem *menuitem, G_GNUC_UNUSED gpointer g
 	gchar	*command = NULL;
 	GError	*error = NULL;
 	GString	*cmd_str = NULL;
-	GtkWidget	*dialog = NULL;
-	GtkWidget 	*label = NULL;
-	GtkWidget 	*entry = NULL;
-	GtkWidget	*vbox = NULL;
 	GKeyFile 	*config = g_key_file_new();
 	gchar 		*config_dir = g_path_get_dirname(config_file);
 	gchar 		*data;
@@ -102,44 +96,22 @@ send_as_attachment(G_GNUC_UNUSED GtkMenuItem *menuitem, G_GNUC_UNUSED gpointer g
 
 			if ((use_address_dialog == TRUE) && (g_strrstr(mailer, "%r") != NULL))
 			{
-				dialog = gtk_dialog_new_with_buttons(_("Recipient's Address"),
+ 				gchar *input = dialogs_show_input(_("Recipient's Address"),
 										GTK_WINDOW(geany->main_widgets->window),
-										GTK_DIALOG_DESTROY_WITH_PARENT,
-										GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-										GTK_STOCK_OK, GTK_RESPONSE_ACCEPT,
-										NULL);
-				gtk_dialog_set_default_response(GTK_DIALOG(dialog), GTK_RESPONSE_ACCEPT);
- 				vbox = ui_dialog_vbox_new(GTK_DIALOG(dialog));
- 				gtk_widget_set_name(dialog, "GeanyDialog");
- 				gtk_box_set_spacing(GTK_BOX(vbox), 10);
+										_("Enter the recipient's e-mail address:"),
+										address);
 
- 				label = gtk_label_new(_("Enter the recipient's e-mail address:"));
- 				gtk_widget_show(label);
- 				gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
- 				entry = gtk_entry_new();
- 				gtk_widget_show(entry);
- 				if (address != NULL)
- 					gtk_entry_set_text(GTK_ENTRY(entry), address);
-
- 				gtk_container_add(GTK_CONTAINER(vbox), label);
- 				gtk_container_add(GTK_CONTAINER(vbox), entry);
- 				gtk_widget_show(vbox);
-
-				g_signal_connect(G_OBJECT(entry), "activate",
-					G_CALLBACK(on_enter_key_pressed_in_entry), dialog);
-
-				if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_ACCEPT)
+				if (input)
 				{
 					g_key_file_load_from_file(config, config_file, G_KEY_FILE_NONE, NULL);
 
 					g_free(address);
- 					address = g_strdup(gtk_entry_get_text(GTK_ENTRY(entry)));
+ 					address = input;
 
  					g_key_file_set_string(config, "tools", "address", address);
  				}
  				else
  				{
-					gtk_widget_destroy(dialog);
 					return;
 				}
 
@@ -169,10 +141,14 @@ send_as_attachment(G_GNUC_UNUSED GtkMenuItem *menuitem, G_GNUC_UNUSED gpointer g
 				if (! utils_string_replace_all(cmd_str, "%r", address))
  					ui_set_statusbar(FALSE,
 					_("Recipient address placeholder not found. The executed command might have failed."));
+					g_free(address);
 			}
 			else
+			{
 				/* Removes %r if option was not activ but was included into command */
 				utils_string_replace_all(cmd_str, "%r", NULL);
+				g_free(address);
+			}
 
 			utils_string_replace_all(cmd_str, "%b", g_path_get_basename(locale_filename));
 
@@ -186,12 +162,6 @@ send_as_attachment(G_GNUC_UNUSED GtkMenuItem *menuitem, G_GNUC_UNUSED gpointer g
 
 			g_free(locale_filename);
 			g_free(command);
-
-			if (dialog != NULL)
-			{
-				gtk_widget_destroy(dialog);
-			}
-
 		}
 		else
 		{
@@ -241,7 +211,7 @@ static void add_stock_item(void)
 }
 
 
-void show_icon()
+static void show_icon()
 {
 	mailbutton = GTK_WIDGET(gtk_tool_button_new_from_stock(GEANYSENDMAIL_STOCK_MAIL));
 	plugin_add_toolbar_item(geany_plugin, GTK_TOOL_ITEM(mailbutton));
@@ -253,7 +223,7 @@ void show_icon()
 	gtk_widget_show_all (mailbutton);
 }
 
-void cleanup_icon()
+static void cleanup_icon()
 {
 	if (mailbutton != NULL)
 	{
@@ -350,7 +320,7 @@ GtkWidget *plugin_configure(GtkDialog *dialog)
 	gtk_widget_show(label2);
 	gtk_misc_set_alignment(GTK_MISC(label2), 0, 0.5);
 
-	pref_widgets.checkbox_icon_to_toolbar = gtk_check_button_new_with_label(_("Showing icon in toolbar"));
+	pref_widgets.checkbox_icon_to_toolbar = gtk_check_button_new_with_label(_("Show toolbar icon"));
 	ui_widget_set_tooltip_text(pref_widgets.checkbox_icon_to_toolbar,
 		_("Shows a icon in the toolbar to send file more easy."));
 	gtk_button_set_focus_on_click(GTK_BUTTON(pref_widgets.checkbox_icon_to_toolbar), FALSE);
@@ -358,7 +328,7 @@ GtkWidget *plugin_configure(GtkDialog *dialog)
 	gtk_widget_show(pref_widgets.checkbox_icon_to_toolbar);
 
 	pref_widgets.checkbox_use_addressdialog = gtk_check_button_new_with_label(_
-		("Using dialog for entering email address of recipients"));
+		("Use dialog for entering email address of recipients"));
 
 	gtk_button_set_focus_on_click(GTK_BUTTON(pref_widgets.checkbox_use_addressdialog), FALSE);
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(pref_widgets.checkbox_use_addressdialog), use_address_dialog);
@@ -380,12 +350,10 @@ GtkWidget *plugin_configure(GtkDialog *dialog)
 void plugin_init(GeanyData G_GNUC_UNUSED *data)
 {
 	GtkTooltips *tooltips = NULL;
-
 	GKeyFile *config = g_key_file_new();
-
 	gchar *kb_label = _("Send file by mail");
-
 	GtkWidget *menu_mail = NULL;
+	GeanyKeyGroup *key_group;
 
 	main_locale_init(LOCALEDIR, GETTEXT_PACKAGE);
 
@@ -417,7 +385,8 @@ void plugin_init(GeanyData G_GNUC_UNUSED *data)
 	g_signal_connect(G_OBJECT(menu_mail), "activate", G_CALLBACK(send_as_attachment), NULL);
 
 	/* setup keybindings */
-	keybindings_set_item(plugin_key_group, SENDMAIL_KB, key_send_as_attachment,
+	key_group = plugin_set_key_group(geany_plugin, "sendmail", COUNT_KB, NULL);
+	keybindings_set_item(key_group, SENDMAIL_KB, key_send_as_attachment,
 		0, 0, "send_file_as_attachment", kb_label, menu_mail);
 
 	gtk_widget_show_all(menu_mail);
