@@ -1,7 +1,7 @@
 /*
  *      geanyvc.c - Plugin to geany light IDE to work with vc
  *
- *      Copyright 2007-2009 Frank Lanitz <frank(at)frank(dot)uvena(dot)de>
+ *      Copyright 2007-2011 Frank Lanitz <frank(at)frank(dot)uvena(dot)de>
  *      Copyright 2007-2009 Enrico Tröger <enrico.troeger@uvena.de>
  *      Copyright 2007 Nick Treleaven <nick.treleaven@btinternet.com>
  *      Copyright 2007-2009 Yura Siamashka <yurand2@gmail.com>
@@ -39,15 +39,19 @@
 #include <gtkspell/gtkspell.h>
 #endif
 
-
-
-PluginFields *plugin_fields;
 GeanyData *geany_data;
 GeanyFunctions *geany_functions;
+GeanyPlugin		*geany_plugin;
 
-PLUGIN_VERSION_CHECK(115);
-PLUGIN_SET_INFO(_("GeanyVC"), _("Interface to different Version Control systems."), VERSION,
-		_("Yura Siamashka <yurand2@gmail.com>,\nFrank Lanitz <frank@frank.uvena.de>"));
+
+PLUGIN_VERSION_CHECK(188);
+PLUGIN_SET_TRANSLATABLE_INFO(
+	LOCALEDIR,
+	GETTEXT_PACKAGE,
+	_("GeanyVC"),
+	_("Interface to different Version Control systems."),
+	VERSION,
+	_("Yura Siamashka <yurand2@gmail.com>,\nFrank Lanitz <frank@frank.uvena.de>"));
 
 /* Some global variables */
 static gboolean set_changed_flag;
@@ -103,6 +107,8 @@ const gchar FILE_STATUS_UNKNOWN[] = "Unknown";
 static GtkWidget *editor_menu_vc = NULL;
 static GtkWidget *editor_menu_commit = NULL;
 static GtkWidget *menu_item_sep = NULL;
+static GtkWidget *menu_entry = NULL;
+
 
 static void registrate();
 static void add_menuitems_to_editor_menu();
@@ -124,8 +130,8 @@ enum
 	COUNT_KB
 };
 
-PLUGIN_KEY_GROUP(geanyvc, COUNT_KB)
-     GSList *get_commit_files_null(G_GNUC_UNUSED const gchar * dir)
+
+GSList *get_commit_files_null(G_GNUC_UNUSED const gchar * dir)
 {
 	return NULL;
 }
@@ -2175,6 +2181,8 @@ static void
 init_keybindings(void)
 {
 	/* init keybindins */
+	GeanyKeyGroup *plugin_key_group;
+	plugin_key_group = plugin_set_key_group(geany_plugin, "geanyvc", COUNT_KB, NULL);
 	keybindings_set_item(plugin_key_group, VC_DIFF_FILE, kbdiff_file, 0, 0,
 			     "vc_show_diff_of_file", _("Show diff of file"), menu_vc_diff_file);
 	keybindings_set_item(plugin_key_group, VC_DIFF_DIR, kbdiff_dir, 0, 0,
@@ -2264,8 +2272,8 @@ plugin_init(G_GNUC_UNUSED GeanyData * data)
 	/* init entries inside editor menu */
 	add_menuitems_to_editor_menu();
 
-	plugin_fields->menu_item = menu_vc;
-	plugin_fields->flags = PLUGIN_IS_DOCUMENT_SENSITIVE;
+	ui_add_document_sensitive(menu_vc);
+	menu_entry = menu_vc;
 }
 
 
@@ -2273,9 +2281,8 @@ plugin_init(G_GNUC_UNUSED GeanyData * data)
 void
 plugin_cleanup()
 {
-	// remove the menu item added in init()
-	gtk_widget_destroy(plugin_fields->menu_item);
 	remove_menuitems_from_editor_menu();
+	gtk_widget_destroy(menu_entry);
 	g_slist_free(VC);
 	VC = NULL;
 	g_free(config_file);
